package de.uni_passau.fim.gitwrapper;

import de.uni_passau.fim.processexecutor.ProcessExecutor;

import java.util.Optional;

/**
 * A {@link MergeStrategy} representing the default 'git merge' merge implementation.
 */
public class DefaultMergeStrategy implements MergeStrategy {

    private final Repository repo;

    /**
     * The default merge strategy that git uses when no
     * <a href="https://git-scm.com/docs/git-merge#_merge_strategies>git merge</a> strategy is specified.
     *
     * @param repo
     *         the repo the strategy operates on
     */
    public DefaultMergeStrategy(Repository repo) {
        this.repo = repo;
    }

    @Override
    public Optional<Integer> merge(Reference left, Reference right) {
        Optional<ProcessExecutor.ExecRes> mergeBase = repo.getGit().exec(repo.getDir(), "merge", "-n", "-q", right.getId());
        return mergeBase.flatMap(res -> repo.getStatus().map(stat -> stat.unmerged.size()));
    }

    @Override
    public String toString() {
        return getClass().getSimpleName();
    }
}
